/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * IP module interface definitions
 */
#ifndef _HS_IP_IF_H
#define _HS_IP_IF_H

#include "..\HsIcmp\hsicmp_if.h"

#define HS_IP_HDR_SZ		20			// IP header size
#define HS_ETH_HDR_SZ		14

typedef void hsip_timer_cb_t(void *arg);
typedef long hsip_timer_start_t(unsigned long timeout_ms, void *arg, hsip_timer_cb_t *cb);
typedef void hsip_timer_stop_t(long timer_han);
typedef void hsip_drv_tx_fn_t(unsigned char *pkt, int len);
typedef void hsip_log_fn_t(char *str);
typedef unsigned long hsip_get_ticks_fn_t(void);

// IP initialization stucture
typedef struct
{
	unsigned long 		local_ip;			// local ip address

	int					local_ip_valid;		// is local IP address valid (upper layer sets to 0 if IP address not known yet)
	
	unsigned long		gateway_ip;			// gateway ip address
	unsigned long		mask;				// local network mask
	unsigned char 		*local_eth_addr;	// pointer to local ethernet address buffer (6 bytes)
	int					mtu;				// maximum transmit unit
	hsip_timer_start_t	*timer_start;		// start timer function
	hsip_timer_stop_t	*timer_stop;		// stop timer function
	hsip_drv_tx_fn_t	*drv_tx;			// physical layer driver transmit function
	hsip_log_fn_t		*log_fn;			// log function for debugging

	hsicmp_event_fn_t	*icmp_event_fn;		// icmp event callback
	hsip_get_ticks_fn_t	*get_ticks;			// get number of millisecond ticks since boot up
} hs_ip_init_t;

typedef struct
{
	unsigned long 		local_ip;			// local ip address
	int 				local_ip_valid;		// is local ip address valid

	unsigned long		gateway_ip;			// gateway ip address
	unsigned long		mask;				// local network mask
} hsip_params_t;

/* Initialise IP stack*/
extern int HsIpInit(hs_ip_init_t *pInit);

/* Set parameters during IP runtime */
extern void HsIpSetParams(hsip_params_t *params);

/* Close IP stack */
extern void HsIpShutdown(void);

/* Send IP packet */
extern int HsIpSendPacket(unsigned long	dest_ip, 		// destination IP address
					unsigned char 		protocol, 		// upper layer protocol
					unsigned char 		*pkt, 			// IP payload
					unsigned short 		length);		// IP payload length

/* Ethernet packet received, not thread safe, MUST NOT be called directly from Interrupt */
extern void HsIpReceivePacket(unsigned char *pkt, int len);

/* calculate IP header checksum */
extern unsigned short HsIpChecksum16(unsigned char* data, int len);

#endif /* _HS_IP_IF_H */
