#ifndef _LIST_H
#define _LIST_H

#ifndef NULL
#define NULL 0
#endif

/* 
 * List structure definitions
 */

/* List item structure */
typedef struct list_t list_t;
typedef struct
{
	void			*next;		// pointer to next element
	void			*prev;		// pointer to previous element
	void			*data;		// data stored in this element
	int				data_size;
	void			*data_ex1;	// additional data 1
	void			*data_ex2;	// additional data 2
	list_t			*list;		// list associated with this element, may be none (NULL)
} list_item_t;

/* List control structure */
typedef struct list_t
{
	list_item_t		*head;		// start of list
	list_item_t		*tail;		// end of list
	list_item_t		*pMarker;	// points to currently processed element	
	list_t 			*pParent;	// pointer on parent list	
} list_t;

/* call back called each time an element of a list is walked */
typedef void 	list_walk_fn_t(list_item_t *p, int i);

 /* Initialise list control data */
void 			ListInit(list_t *ls);

/* Allocate memory for data of specified size and link to list */
list_item_t 	*ListPut(list_t *ls, void *pdata, int size);

/* Allocate memory only for list structure. Use supplied memory for data */
list_item_t 	*ListPutNoDatAlloc(list_t *ls, void *pdata);
 
/* allocate and store pdada of specified size into field data_ex 
 * of an existing item */
void 			ItemAddDataEx1(list_item_t *p, void *pdata, int size);

/* allocate and store pdada of specified size into field data_ex 
 * of an existing item */
void 			ItemAddDataEx2(list_item_t *p, void *pdata, int size);

/* find an item in a list */
list_item_t 	*FindItem(list_t *ls, void *pdata, int size);

/* free list, all items data and all sub-lists */
void			ListFree(list_t *ls);

/* navigate the list and all sub lists */
void 			ListWalk(list_t *ls, list_walk_fn_t *fn);

/* Get first item from the list */
list_item_t *ListWalkGetFirst(list_t *ls);

/* Get next item from the list given the item */
list_item_t *ListWalkGetNext(list_item_t *p);

/* Get last item from the list */
list_item_t *ListWalkGetLast(list_t *ls);

/* Add item to tail */
void ListAddToTail(list_t *ls, list_item_t *p);

/* Get item from head of queue and unlink it (advance head) */
list_item_t *ListGetFromHead(list_t *ls);

/*
 * Delete given item from specified list
 */
void ListDelete(list_t *ls, list_item_t *p);

/* Unlink item from given list without releasing any memory */
void ListUnlink(list_t *ls, list_item_t *p);

#endif /* _LIST_H */
