/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * Socket interface library
 */
#ifndef _HS_UDP_IF
#define _HS_UDP_IF

#define HS_UDP_HDR_SZ				8		// UDP header length

typedef void hs_sock_rxfn_t
(
	unsigned long 		remote_ip,		// remote ip address
	unsigned short int	local_port,		// local port
	unsigned short int	remote_port,	// remote port
	int					protocol,		// protocol
	unsigned char 		*RxBuf,			// received buffer
	unsigned short int 	rxlen			// received length
);

typedef void log_fn_t(char *str);

// UDP parameters
typedef struct
{
	unsigned long local_ip;				// local ip address
} hs_udp_params_t;

/*
 * Initialize UDP module
 * local_ip - Local interface IP address to use
 */
extern void HsUdpInit(unsigned long local_ip, hs_sock_rxfn_t *RxFn, log_fn_t *logfn);

/*
 * Set parameters at run time
 */
extern void HsUdpSetParams(hs_udp_params_t *pParams);

/*
 * Cleanup UDP module
 */
extern void HsUdpCleanUp(void);

/*
 * Send UDP packet to IP payer for transmission
 */
extern
int HsUdpSendPacket(unsigned long 		dest_ip, 		// remote end IP address
					unsigned short int	dest_port,		// remote UDP port number
					unsigned short int	source_port,	// source UDP port number
					unsigned char 		*payload_buf, 	// UDP payload to send
					unsigned short int	payload_len);	// UDP payload length

/* Called from IP module
 * UDP packet received from IP 
 * pUdpPkt - start of UDP packet, contains IP header at offset -20 from this pointer
 * udplen - length of UDP packet
 */
extern void HsUdpReceivePacket(unsigned char *pUdpPkt, int udplen);

#endif /* _HS_UDP_IF */

