/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * HS NTP Protocol Module Demo application
 */
#include "stdafx.h"
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"
#include <stdlib.h>
#include <direct.h>
#include <stdio.h>
#include <process.h>
#include <time.h>
#include <Windowsx.h>
#include <Mmsystem.h>

#include "hsntp_if.h"
#include "hsip_if.h"
#include "hssock_if.h"
#include "hsdns_if.h"
#include "hsdhcp_if.h"
#include "pcap.h"

#define WM_APP_TIMER	WM_USER + 1		// timer messages

#define MAXTIMERS		256

typedef struct
{
	int					allocated;
	int					enabled;
	void 				*arg;
	hs_sock_timer_cb_t 	*cb;
	UINT				timer_id;
} timerbock_t;

#define HS_VERSION	"v1.1"

long start_timer(unsigned long secs, timer_callback_t *callback);
void stop_timer(long handle);

TCHAR			szWindowClass[]		= " ";
HINSTANCE		hInst;
TCHAR			szTitle[] = "HsTCPIPv4 C Source Library: NTP/SNTP demo";

HWND			hwnd_main;
HWND			dlg_main;
HBRUSH			hbrush1				= NULL;

timer_callback_t *ntp_timer_callback = NULL;

char commercial_msg[] = "This is a demo application for HsTCPIPv4 embedded protocol stack. HSNTP is part of HsTCPIPv4\n"
                        "It allows a programmer to receive current time information from any NTP server.\n"
                        "For further information visit: www.hillstone-software.com/hs_tcpipv4_details.htm\n"
                        "email: info@hillstone-software.com";

int session_open = 0;

long num_msgs;
long mbx_size;
long rx_size;
long old_msgs;

long session_handle;

// pcap local interface device list
pcap_if_t *alldevs = NULL;
pcap_t *adhandle = NULL;

static 	HFONT			hfont				= NULL;
static 	LOGFONT			fnt;

int ip_port = DEF_PORT;

unsigned char server_name[256] = "ntp1.belbone.be";

hs_ntp_session_t	ss = {0};
hs_ntp_api_t       api = {0};

unsigned char port_str[5] = {0};
unsigned char regdata[1024] = {0};

conf_t cfg = 
{
	{DEF_PORT},				// port
	{0xC0A80101},			// dest ip address
	{"ntp.server.com"},
};

static COLORREF color;
static int progress = 0;

long update_timer = 0;

unsigned char months[12][4] =
{
	{"Jan"},
	{"Feb"},
	{"Mar"},
	{"Apr"},
	{"May"},
	{"Jun"},
	{"Jul"},
	{"Aug"},
	{"Sep"},
	{"Oct"},
	{"Nov"},
	{"Dec"},
};

static timerbock_t	timers[MAXTIMERS] = {0};

static TIMECAPS		timecaps;

// allocate timer
static timerbock_t *alloc_timer(void)
{
	int i;

	for (i=0; i<MAXTIMERS; i++)
	{
		if (!timers[i].allocated)
		{
			timers[i].allocated = TRUE;
			return &timers[i];
		}
	}

	return NULL;
}

// free timer
static void free_timer(timerbock_t *p)
{
	p->allocated = FALSE;
}

/*
 * Get ASCII month
 */
unsigned char *BlTimeGetMonth(int month)
{
	return &months[month][0];
}

HWND NewDialogWin(HINSTANCE hInst, HWND parent, DLGPROC proc, int dlg_id)
{
	HWND	handle;

	handle = CreateDialog(hInst, (LPCTSTR)dlg_id, parent, proc);

	ShowWindow	(handle, SW_SHOWDEFAULT);
    UpdateWindow(handle);

	return	handle;
}

ATOM BlRegisterClass(HINSTANCE	hInstance,
					UINT		style,
					WNDPROC		proc,
					int			cursor_id,
					HBRUSH		bkg_brush,
					LPCTSTR		lpszClassName,
					int			icon_id,
					int			menu_id)
{
	WNDCLASSEX	wcex;

	wcex.cbSize			= sizeof(WNDCLASSEX); 

	wcex.style			= style;
	wcex.lpfnWndProc	= proc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;

	wcex.hCursor		= 0;
	if (cursor_id)
	{
		wcex.hCursor	= LoadCursor(0, (const char *)cursor_id);
	}

	wcex.hbrBackground	= bkg_brush;
	wcex.lpszMenuName	= (LPCSTR)menu_id;
	wcex.lpszClassName	= lpszClassName;

	wcex.hIconSm		= 0; 
	if (icon_id)
	{
		wcex.hIcon		= LoadIcon(hInstance, (LPCTSTR)icon_id);
	}
	return RegisterClassEx(&wcex);
}

TIMERPROC UpdateTimeDisplay(HWND hwnd,  UINT uMsg, UINT_PTR idEvent,  DWORD dwTime)
{
	SYSTEMTIME tm		= {0};
	unsigned char s[80] = {0};

	if (idEvent != 2)
		return 0;

	GetLocalTime(&tm);

	sprintf(s, "%02d:%02d:%02d\n%02d %s %d",tm.wHour, 
											tm.wMinute, 
											tm.wSecond, 
											tm.wDay, 
											BlTimeGetMonth(tm.wMonth-1),
											tm.wYear);

	SetDlgItemText(dlg_main, IDC_STATIC_LT, s);

	return 0;
}

/*
 * Retrieve configuration from registry, if not found create
 */
void get_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp, size;

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					"Software\\Hillstone Software\\HS NTP C Source Library Demo Application\\1.1",
            		0, KEY_QUERY_VALUE, &hKey );

    if (res == ERROR_SUCCESS)
    {    
		RegQueryValueEx(hKey, "RegData", 0, &dwDisp, (LPBYTE) regdata, &size);

   		memcpy(&cfg, regdata, sizeof(conf_t));
   		ip_port = cfg.port;
   		
		memset(server_name, 0, sizeof server_name);
		memcpy(server_name, cfg.srvname, sizeof cfg.srvname);

    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Hillstone Software\\HS NTP LITE C Source Library Demo Application\\1.1", 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{	
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/*
 * Update configuration in registry
 */
void update_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp;
	BOOL rs = FALSE;

	SendMessage(GetDlgItem(dlg_main, IDC_IPADDRESS2), IPM_GETADDRESS, 0,(LPARAM)&cfg.dest_ip);

	cfg.port = (WORD)GetDlgItemInt(dlg_main, IDC_EDIT_PORT, &rs, FALSE);
	if (!rs) ip_port = DEF_PORT;

	memset(cfg.srvname, 0, sizeof cfg.srvname);
	GetDlgItemText(dlg_main, IDC_EDIT_SRVNAME, cfg.srvname, (sizeof cfg.srvname) - 1);	

	memset(cfg.username, 0, sizeof cfg.username);
    GetDlgItemText(dlg_main, IDC_EDIT_USERNAME, cfg.username, (sizeof cfg.username) - 1);

	memset(cfg.password, 0, sizeof cfg.password);
    GetDlgItemText(dlg_main, IDC_EDIT_PASSWORD, cfg.password, (sizeof cfg.password) - 1);
	

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					"Software\\Hillstone Software\\HS NTP C Source Library Demo Application\\1.1",
            		0, KEY_ALL_ACCESS, &hKey );

    if (res == ERROR_SUCCESS)
    {    
   		memcpy(regdata, &cfg, sizeof(conf_t));
   		RegSetValueEx(hKey, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Hillstone Software\\HS NTP C Source Library Demo Application\\1.1", 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/* Event callback from NTP module */
int ntp_callback(long user_data, int ev, long arg1, long arg2)
{
	unsigned char s[200] = {0};
	unsigned char s1[1024];
	int rc;
	SYSTEMTIME st;
	SYSTEMTIME now;

	struct tm *m;
	long t;
	hs_ntp_info_t *inf;
	long msec;

	switch (ev)
	{
	case HS_NTP_USR_EV_SRVRESP:
		session_open = 0;
		inf = (hs_ntp_info_t *)arg1;

		sprintf(s, "Request Complete. Calculated local clock offset (%ld) secs (%d) milliseconds", 
			inf->offset_seconds, inf->offset_milliseconds);
		color = GREEN;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);

		if (inf->offset_seconds!= 0)
		{
			if (IsDlgButtonChecked(dlg_main, IDC_CHECK1) == BST_CHECKED)
			{
				GetSystemTime(&now);				
				time(&t);

				if (inf->offset_seconds < 0)
				{
					if (now.wMilliseconds >= inf->offset_milliseconds)
					{						
						msec = now.wMilliseconds - inf->offset_milliseconds;
					}
					else
					{
						msec = (1000 - (inf->offset_milliseconds - now.wMilliseconds));
						inf->offset_seconds += -1;
					}
				}
				else
				{
					msec = now.wMilliseconds + inf->offset_milliseconds;
					if (msec >= 1000)
					{
						msec = (msec - 1000);
						inf->offset_seconds += 1;
					}
				}
				
				t += (time_t)inf->offset_seconds;
				m = gmtime(&t);

				st.wYear		= m->tm_year + 1900;
				st.wMonth		= m->tm_mon	 + 1;
				st.wDay			= m->tm_mday;
				st.wDayOfWeek	= m->tm_wday;
				st.wHour		= m->tm_hour;
				st.wMinute		= m->tm_min;
				st.wSecond		= m->tm_sec;
				
				st.wMilliseconds = (WORD)msec;

				SetSystemTime(&st);
			}
		}

		sprintf(s1, "T1 (Time request sent from client)  : %02d:%02d:%02d.%03d\r\n"
					"T2 (Time request received by server): %02d:%02d:%02d.%03d\r\n"
					"T3 (Time reply sent by server)      : %02d:%02d:%02d.%03d\r\n"
					"T4 (Time reply received by client)  : %02d:%02d:%02d.%03d\r\n"
					"Local Clock Offset Requred (secs)   : %ld\r\n",
					inf->st1.wHour, inf->st1.wMinute, inf->st1.wSecond, inf->st1.wMilliseconds, 
					inf->st2.wHour, inf->st2.wMinute, inf->st2.wSecond, inf->st2.wMilliseconds, 
					inf->st3.wHour, inf->st3.wMinute, inf->st3.wSecond, inf->st3.wMilliseconds, 
					inf->st4.wHour, inf->st4.wMinute, inf->st4.wSecond, inf->st4.wMilliseconds, 
					inf->offset_seconds);

		SetDlgItemText(dlg_main, IDC_EDIT_RESP, s1);
		break;

	case HS_NTP_USR_EV_SRVRESPERR:
		session_open = 0;
		rc = (int)arg1;
		sprintf(s, "Invalid Server Reply: (rc=%d): %s", rc, HsNtpGetErrStr(rc));
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
		break;
	
	case HS_NTP_USR_EV_TIMEDOUT:
		session_open = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "No reply from NTP server");		
		break;

	case HS_NTP_USR_EV_DNS_ERROR:
		session_open = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "DNS module cannot resolve NTP server IP address");
		break;

	case HS_NTP_USR_EV_SOCK_ERROR:
		session_open = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "UDP Socket layer error");
		break;		

	case HS_NTP_USR_EV_TMR_ERROR:
		session_open = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "Timer error");
		break;
	}

	return 0;
}

void write_event(char *str)
{
}

static unsigned char hex2byte(unsigned char *buf)
{
	unsigned char hex[] = "0123456789ABCDEF";
	unsigned char b = 0;
    unsigned char b1, b2;
	int i;

    b1 = toupper(buf[0]);
    b2 = toupper(buf[1]);

	for (i = 0; i<16; i++)
	{
		if (b1 == hex[i])
		{
			b &= 0x0f;
			b = (unsigned char)((i << 4) & 0xf0);
			break;
		}
	}

	for (i = 0; i<16; i++)
	{
		if (b2 == hex[i])
		{
			b &= 0xf0;
			b |= (unsigned char)(i & 0x0f);
			break;
		}
	}
	
	return b;
}

// Convert ascii hex ethernet address in the form 00:00:00:00:00:00 to binary 6 byte buffer
static void convert_hex_ethaddr_to_binary(unsigned char *hexbuf, unsigned char *binbuf)
{
	int i, n, maxlen, done;

	maxlen = 17;

	if (strlen(hexbuf) != 17)
		return;

	done = FALSE;
	i = 0; n = 0;
	while (!done)
	{
		binbuf[i] = hex2byte(&hexbuf[n]);

		i++;
		n += 3;

		if (i == 6) 
			done = TRUE;
	}
}

// multimedia timer callback
void CALLBACK mmTimeProc(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	PostMessage(hwnd_main, WM_APP_TIMER, 0, (LPARAM)dwUser);
}

// Process timer message
static void process_timer(LPARAM lParam)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)lParam;

	if (!pTmrBlk) return;

	if (!pTmrBlk->allocated)
		return;

	if (!pTmrBlk->enabled)
		return;

	pTmrBlk->cb(pTmrBlk->arg);

	free_timer(pTmrBlk);
}

// start timer function
static long sock_timer_start(unsigned long timeout_ms, void *arg, hs_sock_timer_cb_t *cb)
{
	MMRESULT res;
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)alloc_timer();
	if (!pTmrBlk) return 0;

	pTmrBlk->arg = arg;
	pTmrBlk->cb = cb;
	pTmrBlk->enabled = TRUE;

	res = timeSetEvent((UINT)timeout_ms, timecaps.wPeriodMin, mmTimeProc, (DWORD)pTmrBlk, TIME_ONESHOT);
	if (res == 0) 
	{	
		free_timer(pTmrBlk);
		return 0;
	}

	pTmrBlk->timer_id = (UINT)res;

	return (long)pTmrBlk;
}

// stop timer function
static void sock_timer_stop(long timer_han)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *) timer_han;

	if (!pTmrBlk) return;

	if (!pTmrBlk->enabled) return;

	timeKillEvent(pTmrBlk->timer_id);
	pTmrBlk->enabled = FALSE;
}

// transmit over physical layer (ethernet)
static void sock_drv_tx_fn(unsigned char *pkt, int len)
{
	if (!adhandle) return;

	/* Send down the packet */
	if (pcap_sendpacket(adhandle,	// Adapter
		pkt,						// buffer with the packet
		len							// size
		) != 0)
	{
		return;
	}
}

// callback used by HSTCP to get number of millisecond ticks since bootup
// Used for generating pseudo random initial sequence number
// if callback pointer not initialized, HSTCP will use default 
// pseudo random number generator to produce initial sequence numbers
static unsigned long hs_sock_get_ticks(void)
{
	return (unsigned long)GetTickCount();
}

// Callback used by the stack whenever it needs a random 32 bit number
static unsigned long hs_sock_get_rand32(void)
{
	return (unsigned long)rand();
}

// global socket events
static long hs_sock_global_event(long user_handle, int ev, long arg1, long arg2, long arg3, long arg4)
{
	hs_dhcp_update_t *p;
	char str[17];

	switch (ev)
	{
	case HS_SOCK_EV_DHCP_UPDATE:
		p = (hs_dhcp_update_t *)arg1;
		if (!p) return 0;

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->local_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS1, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->dns_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_DNS, str);		

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->router_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_GW, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->subnet_mask, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_MASK, str);		
		break;
	}

	return 0;
}

/*
 * Main dialog handler
 */
LRESULT CALLBACK DlgWin(HWND hDlg, UINT message, 
						  WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;
	int rc, inum, i;

	unsigned char ascii_hex_eth[19] = {0};
	unsigned char ethaddr[6];

	unsigned char source_ipaddr[17] = {0};
	unsigned char gw_ipaddr[17] = {0};
	unsigned char dns_ipaddr[17] = {0};
	unsigned char mask[17] = {0};

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	hssock_init_t sockinit = {0};

	switch (message)
	{
	case WM_INITDIALOG:   
        SendDlgItemMessage(hDlg, IDC_EDIT_SRVNAME, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_PORT, EM_LIMITTEXT, 4, 0);
        
        SetDlgItemInt(hDlg, IDC_EDIT_PORT, ip_port, FALSE);
        SetDlgItemText(hDlg, IDC_EDIT_SRVNAME, server_name);        
        
		SendDlgItemMessage(hDlg, IDC_STATIC_CM, WM_SETTEXT, 0, (long)commercial_msg);

		dlg_main = hDlg;
		UpdateTimeDisplay(0, 0, 2, 0);
		
		SendDlgItemMessage(hDlg, IDC_EDIT_RESP, WM_SETFONT, (WPARAM)hfont, TRUE);

		SetDlgItemText(hDlg, IDC_IPADDRESS1, "192.168.1.51");
		SetDlgItemText(hDlg, IDC_EDIT_MAC, "00:1e:68:a4:fb:15");

		SetDlgItemText(hDlg, IDC_IPADDRESS_GW, "192.168.1.254");
		SetDlgItemText(hDlg, IDC_IPADDRESS_MASK, "255.255.255.0");

		SetDlgItemText(hDlg, IDC_IPADDRESS_DNS, "192.168.1.51");

		CheckDlgButton(hDlg, IDC_CHECK_DHCP, BST_CHECKED);
		EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), FALSE);

		/* Retrieve the device list */
		if(pcap_findalldevs(&alldevs, errbuf) == -1)
		{
			sprintf(str, "Error in pcap_findalldevs: %s", errbuf);
			write_event(str);
			return TRUE;
		}

		i = 0;
		for(d=alldevs; d; d=d->next)
		{			
			sprintf(str, "[%d] %s", ++i, d->description);
			SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_ADDSTRING, 0, (long)str);			
		}	

		SendMessage(GetDlgItem(hDlg, IDC_COMBO_ETHIF), CB_SETCURSEL, 0, 0);

		srand( (unsigned)time( NULL ) );
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_CM) || 
			(HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_LT) ||
			((HWND)lParam == GetDlgItem(hDlg, IDC_EDIT_RESP)))
		{
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		else if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_STATUS))
		{
			SetTextColor((HDC)wParam, color);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_CHECK_DHCP:
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED)
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), FALSE);
			else
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), TRUE);
			break;
		
		case IDC_BM_EXIT:
			DestroyWindow(hwnd_main);
			break;

        case IDC_BM_ABORT: 
        	if (!session_open) break;
            break;

		case IDC_INIT:
			if (adhandle)
				pcap_close(adhandle);
			
			inum = (int)SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_GETCURSEL, 0, 0);
			if (inum == CB_ERR)
			{
				SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "Local adapter not selected from list");
				break;
			}

			GetDlgItemText(hDlg, IDC_IPADDRESS1, source_ipaddr, sizeof (source_ipaddr) - 1);
			GetDlgItemText(hDlg, IDC_IPADDRESS_GW, gw_ipaddr, sizeof (gw_ipaddr) - 1);
			GetDlgItemText(hDlg, IDC_IPADDRESS_MASK, mask, sizeof (mask) - 1);
			GetDlgItemText(hDlg, IDC_IPADDRESS_DNS, dns_ipaddr, sizeof (dns_ipaddr) - 1);
			
			GetDlgItemText(hDlg, IDC_EDIT_MAC, ascii_hex_eth, sizeof (ascii_hex_eth) - 1);

			convert_hex_ethaddr_to_binary(ascii_hex_eth, ethaddr);
				
			/* Jump to the selected adapter */
			for(d=alldevs, i=0; i< inum ;d=d->next, i++);

			if ((adhandle= pcap_open_live(d->name,	// name of the device
							 65536,					// portion of the packet to capture. 
													// 65536 grants that the whole packet will be captured on all the MACs.
							 1,						// promiscuous mode (nonzero means promiscuous)
							 1,						// read timeout
							 errbuf					// error buffer
							 )) == NULL)
			{
				SetDlgItemText(dlg_main, 
					IDC_STATIC_STATUS, "Unable to open the adapter : not supported by WinPcap");
				break;
			}

			sockinit.dhcp_client = (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED) ? 1: 0;

			if (sockinit.dhcp_client)
				strcpy(source_ipaddr, "0.0.0.0");

			sockinit.source_ip_str = source_ipaddr;
			sockinit.gateway_ip_str = gw_ipaddr;
			sockinit.dns_server_ip_str = dns_ipaddr;
			sockinit.network_mask = mask;
			sockinit.local_eth = ethaddr;
			sockinit.timer_start_fn = sock_timer_start;
			sockinit.timer_stop_fn = sock_timer_stop;
			sockinit.drv_tx_fn = sock_drv_tx_fn;
			sockinit.log_fn = write_event;
			sockinit.global_ev_fn = hs_sock_global_event;

			sockinit.get_ticks_fn = hs_sock_get_ticks;
			sockinit.seed = (unsigned long)GetTickCount();
			sockinit.get_rand32 = hs_sock_get_rand32;

			rc = HsSockInit(&sockinit);
			if (rc != HS_SOCK_RC_OK)
			{
				sprintf(str, "HsSockInit failed, rc=%s", HsSockGetRcString(rc));
				SetDlgItemText(dlg_main, IDC_STATIC_STATUS, str);
				break;
			}

			SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "HsTCPIPv4 Initialized");

		    api.start_timer = start_timer;
		    api.stop_timer = stop_timer;
		    api.event_cb = ntp_callback;
	
		    rc = HsNtpInit(&api);
			if (rc != HS_NTP_RC_OK)
			{
        		unsigned char s[100] = {0};
		        sprintf(s, "HS NTP Initialisation failure code [%d]", rc);
        
				MessageBox(0, s, 0, 0);
				break;
			}			
			break;

		case IDC_SHUTDOWN:
			HsNtpDestroy();
			
			HsSockCleanUp();
			
			if (adhandle)
			{
				pcap_close(adhandle);
				adhandle = NULL;
			}

			SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "HsTCPIPv4 shutdown");
			break;
            
		case IDC_BM_SEND: 
			if (session_open) break;			
		
            //GetDlgItemText(hDlg, IDC_EDIT_SRVNAME, ss.srv_ip, (sizeof ss.srv_ip) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_SRVNAME, ss.srv_name, (sizeof ss.srv_name) - 1);
			
            GetDlgItemText(hDlg, IDC_EDIT_PORT, port_str, sizeof port_str);
            ss.source_port = (unsigned short)atoi(port_str);
			
			color = 0;
			SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "NTP REQUEST IN PROGRESS");
			
			SetDlgItemText(dlg_main, IDC_EDIT_RESP, "");
	
			rc = HsNtpGetTime(&ss);
			if (rc != HS_NTP_RC_OK)
			{
				unsigned char s[200];
				sprintf(s, "ERROR: HS NTP ERROR (%s)", HsNtpGetErrStr(rc));
				color = RED;
				SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
				break;
			}
			session_open = 1;			
			break;
		}
		
		break; 
	}
    return result;
}

/*
 * Main window message handler
 */
LRESULT CALLBACK WndProc(HWND		hWnd, 
						 UINT		message, 
						 WPARAM		wParam, 
						 LPARAM		lParam)
{
    LVITEM item = {0};

	static INITCOMMONCONTROLSEX icex;

	switch (message) 
	{
	case WM_CREATE:
		icex.dwSize = sizeof(icex);
		icex.dwICC = ICC_DATE_CLASSES;

		InitCommonControlsEx(&icex);

		InitCommonControls(); 

		memset(&fnt, 0, sizeof fnt);
		fnt.lfWidth = 8;
		fnt.lfPitchAndFamily = FF_MODERN;
		hfont	= CreateFontIndirect(&fnt);

		dlg_main = NewDialogWin(hInst, hWnd, DlgWin, IDD_DIALOG_MAIN);
		break;

	case WM_APP_TIMER:
		process_timer(lParam);
		break;

	case WM_DESTROY:
		if (hbrush1)
		{
			SelectObject(GetWindowDC(hWnd), hbrush1);
			DeleteObject(hbrush1);
		}

		if (hfont)
		{
			SelectObject(GetWindowDC(hWnd), (HGDIOBJ)hfont);
			DeleteObject(hfont);
		}

		HsNtpDestroy();

		update_config();

		KillTimer(hWnd, 2);

		HsSockCleanUp();

		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
   
	return 0;
}

/*
 * Init Application Instance
 */
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, HWND *hwnd)
{
	HWND hWnd;

	hInst = hInstance;

	hWnd = CreateWindow(szWindowClass, 
					   szTitle, 
					   WS_OVERLAPPED | WS_SYSMENU,
					   0, 
					   0, 
					   MAINWIDTH,
					   MAINDEPTH, 
					   NULL, LoadMenu(hInstance, 0/*(LPCTSTR)IDR_MENU1*/), 
					   hInstance, NULL);

	if (!hWnd)
	{
		return FALSE;
	}
	*hwnd = hWnd;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;
}

/* start timer */
long start_timer(unsigned long secs, timer_callback_t *callback)
{
	return sock_timer_start(secs * 1000, NULL, callback);
}

/* stop timer */
void stop_timer(long handle)
{
	sock_timer_stop(handle);
}

/* windows timer callback */

/*
 * entry point
 */
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG		msg;
	HACCEL	hAccelTable;
	unsigned char *pkt_data;
	struct pcap_pkthdr *header;
	MMRESULT res;
	int r;

	res = timeGetDevCaps(&timecaps, sizeof (TIMECAPS));
	if (res == TIMERR_NOERROR)
	{			
		timeBeginPeriod((UINT)timecaps.wPeriodMin);
	}

	hbrush1 = CreateSolidBrush(RGB(255, 255, 255));

	get_config();

	BlRegisterClass(hInstance, CS_HREDRAW | CS_VREDRAW, WndProc, (int)IDC_ARROW,
				   (HBRUSH)(COLOR_APPWORKSPACE+1), szWindowClass, IDI_ICON1, 0);

	if (!InitInstance (hInstance, nCmdShow, &hwnd_main)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)0);

	update_timer = (long)SetTimer(hwnd_main, (UINT_PTR)2, 1000, (TIMERPROC)UpdateTimeDisplay);

	while (1) 
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{
				if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
			else
			{
				break;
			}
		}
		else
		{
			Sleep(1);

			if (adhandle)
			{
				r = pcap_next_ex( adhandle, &header, &pkt_data);
				if (r > 0)
				{
					HsIpReceivePacket(pkt_data, (int)header->caplen);					
				}
			}			
		}		
	}

	if (alldevs)
		pcap_freealldevs(alldevs);

	if (adhandle)
		pcap_close(adhandle);

	return msg.wParam;	
}


