#include "stdafx.h"
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"
#include <stdlib.h>
#include <direct.h>
#include <stdio.h>
#include "hssmtp_if.h"
#include <process.h>
#include <Windowsx.h>
#include <Mmsystem.h>
#include "hssmtp_if.h"
#include "hsdhcp_if.h"
#include "hsip_if.h"
#include "hssock_if.h"
#include "pcap.h"

#define WM_APP_TIMER	WM_USER + 1		// timer messages

#define MAXTIMERS		256

typedef struct
{
	int					allocated;
	int					enabled;
	void 				*arg;
	hs_sock_timer_cb_t 	*cb;
	UINT				timer_id;
} timerbock_t;

static timerbock_t	timers[MAXTIMERS] = {0};

// pcap local interface device list
pcap_if_t *alldevs = NULL;
pcap_t *adhandle = NULL;

unsigned char ascii_hex_eth[19] = {0};
unsigned char ethaddr[6];
unsigned char source_ipaddr[17] = {0};
unsigned char gw_ipaddr[17] = {0};
unsigned char mask[17] = {0};

static TIMECAPS timecaps;


TCHAR			szWindowClass[]		= " ";
HINSTANCE		hInst;
TCHAR			szTitle[] = "HsTCPIPv4 C Source Library: SMTP Demo";

HWND			hwnd_main;
HWND			dlg_main;
HBRUSH			hbrush1				= NULL;

char commercial_msg[] = "This is a demo application for HsTCPIPv4 embedded protocol stack. HSSMTP library is part of HsTCPIPv4\n"
                        "It allows a programmer to send email text messages and attachments to remote SMTP server.\n"
                        "Further information: www.hillstone-software.com/hs_tcpipv4_details.htm\n"
                        "email: info@hillstone-software.com";

char *m_title = NULL;
char *m_prmt = NULL;
char *m_err = NULL;
char *m_hdr = "Error";

long session_handle;
long mycontext;

DWORD ip_addr = 0xC0A80101;
int ip_port = DEF_PORT;
unsigned char smtp_server_name[256] = "mail.server.com";
unsigned char smtp_sender[256] = "yourname@your_domain.com";
unsigned char smtp_recv[256] = "receiver@receipient_domain.com";

unsigned char mssg[1024] ={0};

hs_smtp_mail_t	m = {0};
unsigned char srvn[80] = {0};
unsigned char sadr[80] = {0};
unsigned char radr[80] = {0};
unsigned char cc[80] = {0};
unsigned char subj[80] = {0};
unsigned char port_str[5] = {0};
unsigned char regdata[1024] = {0};
unsigned char username[100];
unsigned char password[100];

unsigned char filename[_MAX_PATH];

unsigned char srv_ip[16];

conf_t cfg = 
{
	{25},					// port
	{0xC0A80101},			// dest ip address
	{"mail.server.com"},
};

unsigned char *chrset[] = 
{
	"Default",
	"KOI8-R",
};

static COLORREF color;

// allocate timer
static timerbock_t *alloc_timer(void)
{
	int i;

	for (i=0; i<MAXTIMERS; i++)
	{
		if (!timers[i].allocated)
		{
			timers[i].allocated = TRUE;
			return &timers[i];
		}
	}

	return NULL;
}

// free timer
static void free_timer(timerbock_t *p)
{
	p->allocated = FALSE;
}

static unsigned char hex2byte(unsigned char *buf)
{
	unsigned char hex[] = "0123456789ABCDEF";
	unsigned char b = 0;
    unsigned char b1, b2;
	int i;

    b1 = toupper(buf[0]);
    b2 = toupper(buf[1]);

	for (i = 0; i<16; i++)
	{
		if (b1 == hex[i])
		{
			b &= 0x0f;
			b = (unsigned char)((i << 4) & 0xf0);
			break;
		}
	}

	for (i = 0; i<16; i++)
	{
		if (b2 == hex[i])
		{
			b &= 0xf0;
			b |= (unsigned char)(i & 0x0f);
			break;
		}
	}
	
	return b;
}

void write_event(char *str)
{
	SetDlgItemText(dlg_main, IDC_STATIC_STATUS, str);
}

// Convert ascii hex ethernet address in the form 00:00:00:00:00:00 to binary 6 byte buffer
static void convert_hex_ethaddr_to_binary(unsigned char *hexbuf, unsigned char *binbuf)
{
	int i, n, maxlen, done;

	maxlen = 17;

	if (strlen(hexbuf) != 17)
		return;

	done = FALSE;
	i = 0; n = 0;
	while (!done)
	{
		binbuf[i] = hex2byte(&hexbuf[n]);

		i++;
		n += 3;

		if (i == 6) 
			done = TRUE;
	}
}

// multimedia timer callback
void CALLBACK mmTimeProc(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	PostMessage(hwnd_main, WM_APP_TIMER, 0, (LPARAM)dwUser);
}

// Process timer message
static void process_timer(LPARAM lParam)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)lParam;

	if (!pTmrBlk) return;

	if (!pTmrBlk->allocated)
		return;

	if (!pTmrBlk->enabled)
		return;

	pTmrBlk->cb(pTmrBlk->arg);

	free_timer(pTmrBlk);
}

// start timer function
static long sock_timer_start(unsigned long timeout_ms, void *arg, hs_sock_timer_cb_t *cb)
{
	MMRESULT res;
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)alloc_timer();
	if (!pTmrBlk) return 0;

	pTmrBlk->arg = arg;
	pTmrBlk->cb = cb;
	pTmrBlk->enabled = TRUE;

	res = timeSetEvent((UINT)timeout_ms, timecaps.wPeriodMin, mmTimeProc, (DWORD)pTmrBlk, TIME_ONESHOT);
	if (res == 0) 
	{	
		free_timer(pTmrBlk);
		return 0;
	}

	pTmrBlk->timer_id = (UINT)res;

	return (long)pTmrBlk;
}

// stop timer function
static void sock_timer_stop(long timer_han)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *) timer_han;

	if (!pTmrBlk) return;

	if (!pTmrBlk->enabled) return;

	timeKillEvent(pTmrBlk->timer_id);
	pTmrBlk->enabled = FALSE;
}

// transmit over physical layer (ethernet)
static void sock_drv_tx_fn(unsigned char *pkt, int len)
{
	if (!adhandle) return;

	/* Send down the packet */
	if (pcap_sendpacket(adhandle,	// Adapter
		pkt,						// buffer with the packet
		len							// size
		) != 0)
	{
		return;
	}
}

// callback used by HSTCP to get number of millisecond ticks since bootup
// Used for generating pseudo random initial sequence number
// if callback pointer not initialized, HSTCP will use default 
// pseudo random number generator to produce initial sequence numbers
static unsigned long hs_sock_get_ticks(void)
{
	return (unsigned long)GetTickCount();
}

// Callback used by the stack whenever it needs a random 32 bit number
static unsigned long hs_sock_get_rand32(void)
{
	return (unsigned long)rand();
}

// global socket events
static long hs_sock_global_event(long user_handle, int ev, long arg1, long arg2, long arg3, long arg4)
{
	hs_dhcp_update_t *p;
	char str[17];

	switch (ev)
	{

	// have been aquired by HSDHCP
	case HS_SOCK_EV_DHCP_UPDATE:
		p = (hs_dhcp_update_t *)arg1;
		if (!p) return 0;

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->local_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS1, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->dns_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_DNS, str);		

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->router_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_GW, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->subnet_mask, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_MASK, str);

		break;
	}

	return 0;
}

HWND NewDialogWin(HINSTANCE hInst, HWND parent, DLGPROC proc, int dlg_id)
{
	HWND	handle;

	handle = CreateDialog(hInst, (LPCTSTR)dlg_id, parent, proc);

	ShowWindow	(handle, SW_SHOWDEFAULT);
    UpdateWindow(handle);

	return	handle;
}

// Initialize a combo control	
void InitComboControl(HWND wnd, char **buf, int dglid, int default_ind)
{
	int i = 0;

	while (buf[i])
	{
		SendDlgItemMessage(wnd, dglid, CB_ADDSTRING, 0, (long)buf[i++]);
	}

	SendMessage(GetDlgItem(wnd, dglid), CB_SETCURSEL, default_ind, 0);
}

ATOM BlRegisterClass(HINSTANCE	hInstance,
					UINT		style,
					WNDPROC		proc,
					int			cursor_id,
					HBRUSH		bkg_brush,
					LPCTSTR		lpszClassName,
					int			icon_id,
					int			menu_id)
{
	WNDCLASSEX	wcex;

	wcex.cbSize			= sizeof(WNDCLASSEX); 

	wcex.style			= style;
	wcex.lpfnWndProc	= proc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;

	wcex.hCursor		= 0;
	if (cursor_id)
	{
		wcex.hCursor	= LoadCursor(0, (const char *)cursor_id);
	}

	wcex.hbrBackground	= bkg_brush;
	wcex.lpszMenuName	= (LPCSTR)menu_id;
	wcex.lpszClassName	= lpszClassName;

	wcex.hIconSm		= 0; 
	if (icon_id)
	{
		wcex.hIcon		= LoadIcon(hInstance, (LPCTSTR)icon_id);
	}
	return RegisterClassEx(&wcex);
}

/*
 * Retrieve configuration from registry, if not found create
 */
void get_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp, size;

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					REGKEYNAME,
            		0, KEY_QUERY_VALUE, &hKey );

    if (res == ERROR_SUCCESS)
    {    
		RegQueryValueEx(hKey, "RegData", 0, &dwDisp, (LPBYTE) regdata, &size);

   		memcpy(&cfg, regdata, sizeof(conf_t));
   		ip_addr = cfg.dest_ip;
   		ip_port = cfg.port;
   		
		memset(smtp_server_name, 0, sizeof smtp_server_name);
		memcpy(smtp_server_name, cfg.srvname, sizeof cfg.srvname);

		memset(smtp_sender, 0, sizeof smtp_sender);
		memcpy(smtp_sender, cfg.sadr, sizeof cfg.sadr);

		memset(smtp_recv, 0, sizeof smtp_recv);
		memcpy(smtp_recv, cfg.radr, sizeof cfg.radr);

    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGKEYNAME, 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{	
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/*
 * Update configuration in registry
 */
void update_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp;
	BOOL rs = FALSE;
    unsigned char srvn[80] = {0};

	SendMessage(GetDlgItem(dlg_main, IDC_IPADDRESS2), IPM_GETADDRESS, 0,(LPARAM)&cfg.dest_ip);

	cfg.port = (WORD)GetDlgItemInt(dlg_main, IDC_EDIT_PORT, &rs, FALSE);
	if (!rs) ip_port = DEF_PORT;

	memset(cfg.srvname, 0, sizeof cfg.srvname);
	GetDlgItemText(dlg_main, IDC_EDIT_SRVNAME, cfg.srvname, (sizeof cfg.srvname) - 1);	

	memset(cfg.sadr, 0, sizeof cfg.sadr);
    GetDlgItemText(dlg_main, IDC_EDIT_SENDER, cfg.sadr, (sizeof cfg.sadr) - 1);

	memset(cfg.radr, 0, sizeof cfg.radr);
    GetDlgItemText(dlg_main, IDC_EDIT_RECPT, cfg.radr, (sizeof cfg.radr) - 1);
	

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					REGKEYNAME,
            		0, KEY_ALL_ACCESS, &hKey );

    if (res == ERROR_SUCCESS)
    {    
   		memcpy(regdata, &cfg, sizeof(conf_t));
   		RegSetValueEx(hKey, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGKEYNAME, 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

int smtp_callback(void *session_ref, int ev, long arg1, long arg2)
{
	unsigned char s[300] = {0};
    unsigned char s1[300];
	unsigned char *errstr = (unsigned char *)arg1;
	int len = (int)arg2;
	int i;
    unsigned char *p;

	switch (ev)
	{
	case HS_SMTP_USR_EV_DONE:
		color = GREEN;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "MAIL TRANSFER COMPLETE");
		break;

	case HS_SMTP_USR_EV_TIMEDOUT:
		i = sprintf(s, "ERROR: TIMEOUT WAITING FOR SERVER REPONSE: (%s)", errstr);
		if (((sizeof s) - i-1) < (size_t)len) len = ((sizeof s) - i-1);		
		memcpy(&s[i], errstr, len);
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
		break;

	case HS_SMTP_USR_EV_SRVERR:
        memset(s1, 0, sizeof (s1));
        if (len >= sizeof (s1)) len = (sizeof(s1) - 1);
        memcpy(s1, errstr, len);

        if (strlen(s1) > 1) p = strchr(s1, 13);
        if (p) *p = 0;
        
		sprintf(s, "ERROR: Server Returns Error: %s", s1);
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
		break;

	case HS_SMTP_USR_EV_CONNFAIL:
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "ERROR: TCP CONNECTION REQUEST FAILED");
		break;

	case HS_SMTP_USR_EV_CLOSED:
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "ERROR: TCP SESSION DISCONNECTED");
		break;

	case HS_SMTP_USR_EV_PROGRESS:
	case HS_SMTP_USR_EV_PROGRESS_F:		
		SendMessage(GetDlgItem(dlg_main, IDC_PROGRESS1), PBM_SETPOS, (WPARAM)arg1, 0);
		break;
	}

    return FALSE;
}

/*
 * Ask user for file name
 */
void GetFileName(HWND hDlg)
{
	OPENFILENAME ofn;       // common dialog box structure
	HWND hwnd =NULL;        // owner window

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFile = filename;
	//
	// Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
	// use the contents of szFile to initialize itself.
	//
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(filename);
	ofn.lpstrFilter = "All\0*.*\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	// Display the Open dialog box. 
	if (GetOpenFileName(&ofn)==TRUE) 
	{
		SetDlgItemText(hDlg, IDC_EDIT_FNAME, filename);
	}
}

/*
 * Create file callback
 */
static void *hssmtp_create_file(unsigned char *filename, int mode, int open_always)
{
	HANDLE hFile;
	DWORD accessmode;

	switch (mode)
	{
	case HSSMTP_FILE_READ:
		accessmode = GENERIC_READ;
		break;
			
	case HSSMTP_FILE_WRITE:
		accessmode = GENERIC_WRITE;
		break;

	default:
		return NULL;
	}

	hFile = CreateFile(filename,			// file to open
                 accessmode,				// open for writing
                 0,       					// no sharing
                 NULL,                  	// default security
                 (open_always) ? OPEN_ALWAYS : OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL, 	// normal file
                 NULL); 

	if (hFile == INVALID_HANDLE_VALUE)
		return NULL;

	return (void *)hFile;
}

/*
 * Close file callback
 */
static void hssmtp_close_file(void *hFile)
{
	CloseHandle((HANDLE)hFile);
}

/*
 * Read file callback
 */
static int hssmtp_read_file(void *hFile, void *pData, int length, int *bytes_read)
{
	DWORD dwBytesRead = 0;
	BOOL bResult;

	bResult = ReadFile((HANDLE)hFile, pData, (DWORD)length, &dwBytesRead, NULL);

	*bytes_read = (int)dwBytesRead;

	return (int)bResult;
}

/*
 * Write file callback
 */
static int hssmtp_write_file(void *hFile, void *pData, int length, int *bytes_written)
{
	DWORD dwBytesWritten = 0;
	BOOL bResult;

	bResult = WriteFile((HANDLE)hFile, pData, (DWORD)length, &dwBytesWritten, NULL);

	*bytes_written = (int)dwBytesWritten;

	return (int)bResult;	
}

/*
 * Get file size callback
 */
static void hssmtp_get_filesize(void *hFile, long *fsize)
{
	DWORD dwSizeLow, dwSizeHigh, dwError;
	
    dwSizeLow = GetFileSize ((HANDLE)hFile, &dwSizeHigh);          
    if (dwSizeLow == 0xFFFFFFFF && 
        (dwError = GetLastError()) != NO_ERROR )
    { 
    	*fsize = 0;
		return; 
    }     

	*fsize = (long)((dwSizeHigh * MAXDWORD) + dwSizeLow); 
}

// Initialize
static void initialize(HWND hDlg)
{
	int rc, inum, i;
	hssmtp_init_t smtp_init = {0};

	unsigned char ascii_hex_eth[19] = {0};
	unsigned char ethaddr[6];

	unsigned char source_ipaddr[17] = {0};
	unsigned char gw_ipaddr[17] = {0};
	unsigned char mask[17] = {0};
	unsigned char dns_ipaddr[17] = "0.0.0.0";

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	hssock_init_t sockinit = {0};

	if (adhandle)
		pcap_close(adhandle);
			
	inum = (int)SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_GETCURSEL, 0, 0);
	if (inum == CB_ERR)
	{
		write_event("Local adapter not selected from list");
		return;
	}

	GetDlgItemText(hDlg, IDC_IPADDRESS1, source_ipaddr, sizeof (source_ipaddr) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_GW, gw_ipaddr, sizeof (gw_ipaddr) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_MASK, mask, sizeof (mask) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_DNS, dns_ipaddr, sizeof (dns_ipaddr) - 1);		
	GetDlgItemText(hDlg, IDC_EDIT_MAC, ascii_hex_eth, sizeof (ascii_hex_eth) - 1);

	convert_hex_ethaddr_to_binary(ascii_hex_eth, ethaddr);
				
	/* Jump to the selected adapter */
	for(d=alldevs, i=0; i< inum ;d=d->next, i++);

	if ((adhandle= pcap_open_live(d->name,	// name of the device
					 65536,					// portion of the packet to capture. 
											// 65536 grants that the whole packet will be captured on all the MACs.
					 1,						// promiscuous mode (nonzero means promiscuous)
					 1,						// read timeout
					 errbuf					// error buffer
					 )) == NULL)
	{
		write_event("Unable to open the adapter : not supported by WinPcap");
		return;
	}

	sockinit.dhcp_client = (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED) ? 1: 0;

	if (sockinit.dhcp_client)
		strcpy(source_ipaddr, "0.0.0.0");

	sockinit.source_ip_str = source_ipaddr;
	sockinit.gateway_ip_str = gw_ipaddr;
	sockinit.network_mask = mask;
	sockinit.dns_server_ip_str = dns_ipaddr;	
	sockinit.local_eth = ethaddr;
	sockinit.timer_start_fn = sock_timer_start;
	sockinit.timer_stop_fn = sock_timer_stop;
	sockinit.drv_tx_fn = sock_drv_tx_fn;
	sockinit.log_fn = write_event;

	sockinit.get_ticks_fn = hs_sock_get_ticks;
	sockinit.seed = (unsigned long)GetTickCount();
	sockinit.get_rand32 = hs_sock_get_rand32;
	sockinit.global_ev_fn = hs_sock_global_event;

	rc = HsSockInit(&sockinit);
	if (rc != HS_SOCK_RC_OK)
	{
		sprintf(str, "HsSockInit failed, rc=%s", HsSockGetRcString(rc));
		write_event(str);
		return;
	}

	strcpy(smtp_init.domain, "mydomain");
	smtp_init.start_timer = sock_timer_start;
	smtp_init.stop_timer = sock_timer_stop;

	smtp_init.create_file = hssmtp_create_file;
	smtp_init.close_file = hssmtp_close_file;
	smtp_init.read_file = hssmtp_read_file;
	smtp_init.write_file = hssmtp_write_file;
	smtp_init.getticks = hs_sock_get_ticks;
	smtp_init.getfile_size = hssmtp_get_filesize;

	if (HsSmtpInit(&smtp_init) != HS_SMTP_RC_OK)
	{
		MessageBox(0,"HS SMTP Initialisation failure", 0, 0);
		return;
	}

	write_event("HsTCPIPv4 Initialized");
}

/*
 * Main dialog handler
 */
LRESULT CALLBACK DlgWin(HWND hDlg, UINT message, 
						  WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;
	int rc, i;
    char windir[_MAX_PATH] = {0};
    char expl_path[_MAX_PATH] = {0};
    int auth_checked = FALSE;

	unsigned char ascii_hex_eth[19] = {0};

	unsigned char source_ipaddr[17] = {0};
	unsigned char gw_ipaddr[17] = {0};
	unsigned char dns_ipaddr[17] = {0};
	unsigned char mask[17] = {0};

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	hssock_init_t sockinit = {0};
    
	switch (message)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_IPADDRESS1, "192.168.1.51");
		SetDlgItemText(hDlg, IDC_EDIT_MAC, "00:1e:68:a4:fb:15");

		SetDlgItemText(hDlg, IDC_IPADDRESS_GW, "192.168.1.254");
		SetDlgItemText(hDlg, IDC_IPADDRESS_MASK, "255.255.255.0");

		SetDlgItemText(hDlg, IDC_IPADDRESS_DNS, "192.168.1.51");
		
		CheckDlgButton(hDlg, IDC_CHECK_DHCP, BST_CHECKED);
		EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), FALSE);
		
        SendDlgItemMessage(hDlg, IDC_EDIT_SRVNAME, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_SENDER, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_RECPT, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_CC, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_SUBJ, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_PORT, EM_LIMITTEXT, 4, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_MSG, EM_LIMITTEXT, (sizeof mssg)-1, 0);

        
        SetDlgItemInt(hDlg, IDC_EDIT_PORT, ip_port, FALSE);
        SetDlgItemText(hDlg, IDC_EDIT_SRVNAME, smtp_server_name);
        SetDlgItemText(hDlg, IDC_EDIT_SENDER, smtp_sender);
        SetDlgItemText(hDlg, IDC_EDIT_RECPT, smtp_recv);
        SetDlgItemText(hDlg, IDC_EDIT_SUBJ, "HS SMTP C Library Test");
        SetDlgItemText(hDlg, IDC_EDIT_MSG, "<Type Your Message Here ...>");
        
		SendMessage(GetDlgItem(hDlg, IDC_IPADDRESS2), IPM_SETADDRESS, 0, (LPARAM)ip_addr);
        
		SendDlgItemMessage(hDlg, IDC_STATIC_CM, WM_SETTEXT, 0, (long)commercial_msg);

		SendDlgItemMessage(hDlg, IDC_RADIO_NAME, BM_SETCHECK, BST_CHECKED, 0);

		InitComboControl(hDlg, chrset, IDC_COMBO_CHRSET, 0);

		/* Retrieve the device list */
		if(pcap_findalldevs(&alldevs, errbuf) == -1)
		{
			sprintf(str, "Error in pcap_findalldevs: %s", errbuf);
			write_event(str);
			return TRUE;
		}

		i = 0;
		for(d=alldevs; d; d=d->next)
		{			
			sprintf(str, "[%d] %s", ++i, d->description);
			SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_ADDSTRING, 0, (long)str);			
		}	

		SendMessage(GetDlgItem(hDlg, IDC_COMBO_ETHIF), CB_SETCURSEL, 0, 0);

		srand( (unsigned)time( NULL ) );			
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_CM))
		{
			//SetTextColor((HDC)wParam, RED);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		else if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_STATUS))
		{
			SetTextColor((HDC)wParam, color);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_CHECK_DHCP:
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED)
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), FALSE);
			else
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), TRUE);
			break;

		case IDC_INIT:
			initialize(hDlg);
			break;

		case IDC_SHUTDOWN:
			HsSmtpDestroy();

			HsSockCleanUp();

			if (adhandle)
			{
				pcap_close(adhandle);
				adhandle = NULL;
			}

			write_event("HsTCPIPv4 De-Initialized");
			break;

        case IDC_BM_ABORT: 
            if (HsSmtpAbortMail(session_handle) == HS_SMTP_RC_OK)
            {
                color = RED;
		        SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "ABORTING MAIL SESSION");
            }
            break;
            
		case IDC_BM_SEND:
			memset(srvn, 0, sizeof(srvn));
			memset(sadr, 0, sizeof(sadr));
			memset(radr, 0, sizeof(radr));
			memset(cc, 0, sizeof(cc));
			memset(subj, 0, sizeof(subj));
			memset(port_str, 0, sizeof(port_str));
			memset(mssg, 0, sizeof(mssg));

            GetDlgItemText(hDlg, IDC_EDIT_SRVNAME, srvn, (sizeof srvn) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_SENDER, sadr, (sizeof sadr) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_RECPT, radr, (sizeof radr) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_CC, cc, (sizeof cc) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_SUBJ, subj, (sizeof subj) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_PORT, port_str, 4);			
	        GetDlgItemText(hDlg, IDC_EDIT_MSG, mssg, sizeof (mssg)-1);

            memset(&m, 0, sizeof (hs_smtp_mail_t));

			m.srv_name = srvn;
			m.srv_port = (unsigned short)atoi(port_str);
			m.send_addr = sadr;
			m.recv_addr = radr;
			m.subj = subj;
			m.msgp = mssg;
			m.msglen = strlen(mssg);
			m.cc = cc;

			m.session_ref = (void *)mycontext;
			
			m.callback = smtp_callback;

            if (IsDlgButtonChecked(hDlg, IDC_CHECK_AUTH) == BST_CHECKED)
            {
                m.do_auth = TRUE;
                memset(username, 0, sizeof username);
                memset(password, 0, sizeof password);
                GetDlgItemText(hDlg, IDC_EDIT_USR, username, (sizeof username)-1);
                GetDlgItemText(hDlg, IDC_EDIT_PWD, password, (sizeof password)-1);
                m.usrname = username;
                m.password = password;
            }

			memset(filename, 0, sizeof (filename));
			GetDlgItemText(hDlg, IDC_EDIT_FNAME, filename, sizeof (filename)-1);

			if (strlen(filename) != 0)
			{
				m.send_file = TRUE;
				m.filename = filename;
				m.use_latin1 = TRUE;
			}
			else
			{
				m.use_latin1 = FALSE;
			}

			i = (int)SendDlgItemMessage(dlg_main, IDC_COMBO_CHRSET, CB_GETCURSEL, 0, 0);
			if (i == CB_ERR) i=0;

			m.subject_charset = i;

			color = 0;
			SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "INITIATING MAIL SESSION");

			SendMessage(GetDlgItem(dlg_main, IDC_PROGRESS1), PBM_SETPOS, (WPARAM)0, 0);

			rc = HsSmtpSendMail(&m, &session_handle);
			if (rc != HS_SMTP_RC_OK)
			{
				unsigned char s[80];
				sprintf(s, "ERROR: HS SMTP ERROR (%d)", rc);
				color = RED;
				SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
			}
			break;

        case IDC_CHECK_AUTH:
            if (IsDlgButtonChecked(hDlg, IDC_CHECK_AUTH) == BST_CHECKED)
                auth_checked = TRUE;      

            EnableWindow(GetDlgItem(hDlg, IDC_EDIT_USR), auth_checked);
            EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PWD), auth_checked);
            EnableWindow(GetDlgItem(hDlg, IDC_STATIC_USR), auth_checked);
            EnableWindow(GetDlgItem(hDlg, IDC_STATIC_PWD), auth_checked);  

            if (auth_checked)
            {
                SendDlgItemMessage(hDlg, IDC_EDIT_USR, EM_LIMITTEXT, (sizeof username) -1, 0);
                SendDlgItemMessage(hDlg, IDC_EDIT_PWD, EM_LIMITTEXT, (sizeof password) -1, 0);
            }
            break;

		case IDC_BUTTON_FILE:
			GetFileName(hDlg);
			break;
		}
		break; 
	}
    return result;
}

/*
 * Main window message handler
 */
LRESULT CALLBACK WndProc(HWND		hWnd, 
						 UINT		message, 
						 WPARAM		wParam, 
						 LPARAM		lParam)
{
	static INITCOMMONCONTROLSEX icex;

	switch (message) 
	{
	case WM_CREATE:
		icex.dwSize = sizeof(icex);
		icex.dwICC = ICC_DATE_CLASSES;

		InitCommonControlsEx(&icex);

		InitCommonControls(); 

		dlg_main = NewDialogWin(hInst, hWnd, DlgWin, IDD_DIALOG_MAIN);
		break;

	case WM_APP_TIMER:
		process_timer(lParam);
		break;

	case WM_DESTROY:
		if (hbrush1)
		{
			SelectObject(GetWindowDC(hWnd), hbrush1);
			DeleteObject(hbrush1);
		}

		HsSmtpDestroy();

		HsSockCleanUp();

		update_config();

		if (alldevs)
		{
			pcap_freealldevs(alldevs);
			alldevs = NULL;
		}

		if (adhandle)
		{
			pcap_close(adhandle);
			adhandle = NULL;
		}

		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
   
	return 0;
}

/*
 * Init Application Instance
 */
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, HWND *hwnd)
{
	HWND hWnd;

	hInst = hInstance;

	hWnd = CreateWindow(szWindowClass, 
					   szTitle, 
					   WS_OVERLAPPED | WS_SYSMENU,
					   0, 
					   0, 
					   MAINWIDTH,
					   MAINDEPTH, 
					   NULL, LoadMenu(hInstance, 0/*(LPCTSTR)IDR_MENU1*/), 
					   hInstance, NULL);

	if (!hWnd)
	{
		return FALSE;
	}
	*hwnd = hWnd;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;
}

/*
 * entry point
 */
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG		msg;
	HACCEL	hAccelTable;
	unsigned char *pkt_data;
	struct pcap_pkthdr *header;
	int r;

	hbrush1 = CreateSolidBrush(RGB(255, 255, 255));

	get_config();

	BlRegisterClass(hInstance, CS_HREDRAW | CS_VREDRAW, WndProc, (int)IDC_ARROW,
				   (HBRUSH)(COLOR_APPWORKSPACE+1), szWindowClass, IDI_ICON1, 0/*IDR_MENU1*/);

	if (!InitInstance (hInstance, nCmdShow, &hwnd_main)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)0);

	// main message processing loop
	while (1) 
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{
				if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
			else
			{
				break;
			}
		}
		else
		{
			HsSmtpTick();
		
			Sleep(1);

			if (adhandle)
			{
				r = pcap_next_ex( adhandle, &header, &pkt_data);
				if (r > 0)
				{
					HsIpReceivePacket(pkt_data, (int)header->caplen);					
				}
			}			
		}		
	}

	if (alldevs)
		pcap_freealldevs(alldevs);

	if (adhandle)
		pcap_close(adhandle);

	return msg.wParam;	
}


